#!/usr/bin/perl -w
# vim:ts=4

use strict;
use Nagios::ActivityLog; # Steve's module for .log files
use Nagios::Config;   # CPAN module for .cfg files
use Time::Local;
use CGI;

#########################################################################
my($HTMLDOC) = "/usr/bin/htmldoc";
my($CFGFILE) = "/u01/nagios/etc/nagios.cfg";
my($TMP)     = "/tmp";
my($WEBROOT) = "/u01/www/html";
#########################################################################
my($cfg, $log, $entries, $filter, $xfilter, $recordset, $ssummary, $hsummary);
my($hostgroup) = '';
my(@hosts) = ();
my($stime,$etime);
my(@ltime);
my($hrset, $srset);
my($svc,@services);
my($i);
my($q) = new CGI;

# Percentages
my($u,$sd,$ud,$un,$m) = (0,0,0,0,0);
my($o,$sw,$uw,$sc,$uc,$su,$uu) = (0,0,0,0,0,0,0);
my($k,$bgc);

my($thf,$thb)      = ("#ffffcc","#888888");
my($tf,$tb)        = ("#000000","#ffffff");
my($tdhsb)         = ("#cccccc");
my($tdob)          = ("#80ff80");
my($tdswb, $tduwb) = ("#ffffcc","#ffff80");
my($tdscb, $tducb) = ("#ffcccc","#cc8080");
my($tdsub, $tduub) = ("#ffeeaa","#ffcc80");

sub fmt($) {
	my($sec) = $_[0];
	my($rv) = "";
	my($v);

	return '' if(!$sec);
#	return '<div align=right>0s</div>' if(!$sec);
	if($sec > (3600*24)) {
		# days
		$v = int($sec/(3600*24));
		if($v) { $rv .= ' '.$v.'d' ; $sec -= $v*3600*24; }
	}
	if($sec > 3600) {
		# hours
		$v = int($sec/3600);
		if($v) { $rv .= ' '.$v.'h' ; $sec -= $v*3600; }
	}
	if($sec > 60) {
		# min
		$v = int($sec/60);
		if($v) { $rv .= ' '.$v.'m' ; $sec -= $v*60; }
	}
	# sec
	$rv .= ' '.int($sec).'s' if($sec);
	$rv = '0s' if(!$rv);

	return '<DIV align=right>'.$rv.'</div>';
}

sub inlist {
	my($v) = shift;
	return 0 if(!defined $v);
	foreach (@_) { return 1 if($v eq $_); }
	return 0;
}
sub denull($) { return 0 if(!$_[0]); return $_[0]; }
sub dp($) { 
	my($v) = $_[0];
	$v = (int($v * 100 +0.49)/100); 
	if(!$v)  {
		$v="<CENTER>-</CENTER>";
	} else {
		$v = "<div align=right>".$v."%</div>"; 
	}
	return $v;
}
sub col($$$) {
	my($r,$g,$b);
	my($c,$w,$u) = @_;
	my($o);

	$o = 100 - $c - $w - $u;

	$r = int(( $o*204 + $c * 255 + $w * 255 + $u * 255)/100);
	$g = int(( $o*255 + $c * 64  + $w * 255 + $u * 204)/100);
	$b = int(( $o*204 + $c * 64  + $w *  64 + $u * 128)/100);
#	$b = 128;

	return (sprintf "#%02x%02x%02x",$r,$g,$b);
}
# Host split
sub hsplit($) {
	my($h) = $_[0];
	my($u,$sd,$ud,$un,$m) = (0,0,0,0,0);
	my($t) = 0;

	$u = denull($h->{UP}) + denull($h->{'UP:S'});
	$sd = denull($h->{'DOWN:S'});
	$ud = denull($h->{DOWN});
	$un = denull($h->{UNKNOWN}) + denull($h->{'UNKNOWN:S'});
	$m = denull($h->{MISSING});
	$t = $u + $sd + $ud + $un + $m;
	if($t>0) {
		$u = ($u*100)/$t; $sd= ($sd*100)/$t; $ud= ($ud*100)/$t;
		$un= ($un*100)/$t; $m = ($m*100)/$t;
	}
	return ($u,$sd,$ud,$un,$m);
}
# Service split
sub ssplit($) {
	my($s) = $_[0];
	my($o,$sw,$uw,$sc,$uc,$su,$uu,$m) = (0,0,0,0,0,0,0,0);
	my($t) = 0;

	$o = denull($s->{OK}) + denull($s->{'OK:S'}) + denull($s->{'OK:H'});
	$uw = denull($s->{WARNING});
	$sw = denull($s->{'WARNING:S'}) + denull($s->{'WARNING:H'});
	$uc = denull($s->{CRITICAL});
	$sc = denull($s->{'CRITICAL:S'}) + denull($s->{'CRITICAL:H'});
	$uu = denull($s->{UNKNOWN});
	$su = denull($s->{'UNKNOWN:S'}) + denull($s->{'UNKNOWN:H'});
	$m = denull($s->{MISSING});
	
	$t = $o+$sw+$uw+$sc+$uc+$su+$uu+$m;
	if($t>0) {
		$o = ($o *100)/$t; $sw= ($sw*100)/$t; $uw= ($uw*100)/$t;
		$sc= ($sc*100)/$t; $uc= ($uc*100)/$t; $su= ($su*100)/$t;
		$uu= ($uu*100)/$t; $m = ($m *100)/$t;
	}
	return ($o,$sw,$uw,$sc,$uc,$su,$uu,$m);
}

#####################################################################
sub hsreport($) {
	my($ssummary) = $_[0];
	my($rv) = '';

$rv .= $q->h1("HOST STATUS INFORMATION:")."\n";
$rv .= "<TABLE border=0 cellpadding=0 cellspacing=1 color=$tf bgcolor=$tb>\n";
$rv .= "<TR><TD rowspan=2 bgcolor=$thb color=$thf align=center>Host</TD><TD rowspan=2 bgcolor=$thb color=$thf align=center>Uptime</TD><TD colspan=2 bgcolor=$thb color=$thf align=center>Downtime</TD><TD rowspan=2 bgcolor=$thb color=$thf align=center>Unknown</TD><TD rowspan=2 bgcolor=$thb color=$thf align=center>Missing</TD></TR>\n";
$rv .= "<TR><TD bgcolor=$thb color=$thf align=center>Unsched</TD><TD bgcolor=$thb color=$thf align=center>Sched</TD></TR>\n";
foreach $k ( sort keys %$ssummary ) {
	next if($ssummary->{$k}->{type} ne 'HOST');
	($u,$sd,$ud,$un,$m) = hsplit($ssummary->{$k});
	$bgc = col($ud,0,$un);
	$rv .= "<TR><TD bgcolor=$bgc>$k</TD><TD bgcolor=$tdob>"
		.dp($u).fmt($ssummary->{$k}{'UP'})."</TD><TD bgcolor=$tducb>"
		.dp($ud).fmt($ssummary->{$k}{'DOWN'})."</TD><TD bgcolor=$tdscb>"
		.dp($sd).fmt($ssummary->{$k}{'DOWN:S'})."</TD><TD bgcolor=$tdsub>"
		.dp($un).fmt($ssummary->{$k}{'UNKNOWN'})."</TD><TD bgcolor=$tdhsb>"
		.dp($m).fmt($ssummary->{$k}{'MISSING'})."</TD></TR>\n";
}
$rv .= "</TABLE>\n";
return $rv;
}
################################################################
sub ssreport($) {
	my($ssummary)=$_[0];
	my($rv) = '';
	my($ch,$cs,$lh) = ("","","");

$rv .= $q->h1("SERVICE STATUS INFORMATION:")."\n";
$rv .= "<TABLE border=0 cellpadding=0 cellspacing=1>\n";
$rv .= "<TR><TD rowspan=2 bgcolor=$thb color=$thf align=center>Host</TD><TD rowspan=2 bgcolor=$thb color=$thf align=center>Service</TD><TD rowspan=2 bgcolor=$thb color=$thf align=center>OK</TD><TD colspan=2 bgcolor=$thb color=$thf align=center>Warning</TD><TD colspan=2 bgcolor=$thb color=$thf align=center>Critical</TD><TD colspan=2 bgcolor=$thb color=$thf align=center>Unknown</TD><TD rowspan=2 bgcolor=$thb color=$thf align=center>Missing</TD></TR>\n";
$rv .= "<TR><TD bgcolor=$thb color=$thf align=center>Unsched</TD><TD bgcolor=$thb color=$thf align=center>Sched</TD><TD bgcolor=$thb color=$thf align=center>Unsched</TD><TD bgcolor=$thb color=$thf align=center>Sched</TD><TD bgcolor=$thb color=$thf align=center>Unsched</TD><TD bgcolor=$thb color=$thf align=center>Sched</TD></TR>\n";
foreach $k ( sort keys %$ssummary ) {
	next if($ssummary->{$k}->{type} ne 'SERVICE');
	($o,$sw,$uw,$sc,$uc,$su,$uu,$m) = ssplit($ssummary->{$k});
	$k =~ /(.+);(.+)/;
	($ch,$cs) = ($1,$2);
	if($ch ne $lh) { $lh = $ch; } else { $ch = ""; }
	$bgc = col($uc,$uw,$uu);
	$rv .= "<TR><TD bgcolor=$tdhsb>$ch</TD><TD bgcolor=$bgc>$cs</TD><TD bgcolor=$tdob>".dp($o)."</TD><TD bgcolor=$tduwb>".dp($uw)."</TD><TD bgcolor=$tdswb>".dp($sw)."</TD><TD bgcolor=$tducb>".dp($uc)."</TD><TD bgcolor=$tdscb>".dp($sc)."</TD><TD bgcolor=$tduub>".dp($uu)."</TD><TD bgcolor=$tdsub>".dp($su)."</TD><TD bgcolor=$tdhsb>".dp($m)."</TD></TR>\n";
}
$rv .= "</TABLE>\n";
return $rv;
}
##############################################################
sub alertlog($) {
	my($ssummary) = $_[0];
	my($rv) = '';

$rv .= $q->h1("ALERT HISTORY:")."\n";
$rv .= "<TABLE border=0 cellspacing=1><TR><TD></TD><TD>Time</TD><TD>Host</TD><TD>Service</TD><TD>Information</TD></TR>\n";
my($type);
foreach my $r ( @$recordset ) {
	next if(!$r); # null record??!
	next if($r->{'time'}<$stime); # too early
	$bgc="#ffffff";
	if( $r->{type} =~ /DOWNTIME/ ) {
		$bgc = "#ffffff"; $type= "SCHED";
	} elsif ( $r->{state} =~ /WARN/ ) {
		$bgc = "#ffffcc"; $type= "WARN";
	} elsif ( $r->{state} =~ /CRIT/ ) {
		$bgc = "#ffcccc"; $type= "CRIT";
	} elsif ( $r->{state} =~ /UN/ ) {
		$bgc = "#ffeeaa"; $type= "UNKN";
	} else {
		next; # not interested unless a warn, crit or downtime
	}
	$rv .= "<TR><TD bgcolor=#ffffff>".$type
		."</TD><TD bgcolor=$bgc>".localtime($r->{'time'})
		."</TD><TD bgcolor=$bgc>".$r->{host}."</TD><TD bgcolor=$bgc>";
	$rv .= $r->{service} if(defined $r->{service});
	$rv .= "</TD><TD bgcolor=$bgc>".$r->{text}."</TD></TR>\n";
}
$rv .= "</TABLE>\n";
return $rv;
}

#######################################################################
sub hgreport($) {
my($hostgroup) = $_[0];
my($ch,$cs,$lh) = ("","","");

#print "Creating Log object and Config object<BR>\n";
$log = Nagios::ActivityLog->new($CFGFILE);
$cfg = Nagios::Config->new($CFGFILE);

# How to retrieve a hostgroup
foreach my $hg ($cfg->list_hostgroups()) {
	if( $hg->name eq $hostgroup ) {
		@hosts = (split /,/,$hg->members);
		last;
	}
}
if(!@hosts) { print "Unable to find hostgroup $hostgroup<BR>\n"; }
$i=0;
foreach $svc ($cfg->list_services()) {
	if( inlist($svc->{host_name},@hosts) ) {
		push @services, ($svc->{host_name}.';'.$svc->name);
		$i+=1;
	}
}

@ltime = localtime(time);
if( $q->param('T') == -1 ) {
$etime = timelocal(0,0,0,1,$ltime[4],$ltime[5]); # beginning of this month
if($ltime[4]) { $ltime[4] -= 1; }
else { $ltime[4] = 11; $ltime[5] -= 1; }
$stime = timelocal(0,0,0,1,$ltime[4],$ltime[5]); # beginning of prev month
} else {
$stime = timelocal(0,0,0,1,$ltime[4],$ltime[5]); # beginning of this month
$etime = time;
}

print "Logs start on ".(localtime($log->logstart))."<BR>\n";
print "Fetching logs for current month for hostgroup [$hostgroup]<BR>\n";
print "From ".localtime($stime)."\nTo ".localtime($etime)."<BR>\n";

print $q->a({href=>($q->url()."?P=PDF&HG=$hostgroup&T=".$q->param('T'))},
	"Show this as a PDF report").$q->br."\n";

# We make a filter for  what we want
$filter = Nagios::ActivityLog::Filter->new(
	end=>$etime, start=>$stime, host=>[@hosts]);
# We actually make a filter for 1 week earlier
$xfilter = Nagios::ActivityLog::Filter->new(
	end=>$etime, start=>($stime-(24*3600*7)), host=>[@hosts]);
$recordset = $log->fetch($xfilter);
print "We have fetched ".($#$recordset + 1)." records.<BR>\n";

#$hsummary = $hrset->summary({start=>$stime,end=>$etime,hosts=>[@hosts]});
$ssummary = $recordset->summary($filter);

print hsreport($ssummary);
print ssreport($ssummary);
print alertlog($recordset) if($q->param('ALT'));

}

sub makepdf($) {
	my($cmd,@files);
	my($tmpdir) = "$TMP/html.$$";

	$log = Nagios::ActivityLog->new($CFGFILE);
	$cfg = Nagios::Config->new($CFGFILE);

# How to retrieve a hostgroup
	foreach my $hg ($cfg->list_hostgroups()) {
		if( $hg->name eq $hostgroup ) {
			@hosts = (split /,/,$hg->members);
			last;
		}
	}
	if(!@hosts) { print "Unable to find hostgroup $hostgroup<BR>\n"; }
	$i=0;
	foreach $svc ($cfg->list_services()) {
		if( inlist($svc->{host_name},@hosts) ) {
			push @services, ($svc->{host_name}.';'.$svc->name);
			$i+=1;
		}
	}

	@ltime = localtime(time);
	if( $q->param('T') == -1 ) {
		$etime = timelocal(0,0,0,1,$ltime[4],$ltime[5]); # 
		if($ltime[4]) { $ltime[4] -= 1; }
		else { $ltime[4] = 11; $ltime[5] -= 1; }
		$stime = timelocal(0,0,0,1,$ltime[4],$ltime[5]); #
	} else {
		$stime = timelocal(0,0,0,1,$ltime[4],$ltime[5]); # 
		$etime = time;
	}

# We make a filter for  what we want
	$filter = Nagios::ActivityLog::Filter->new(
		end=>$etime, start=>$stime, host=>[@hosts]);
	$xfilter = Nagios::ActivityLog::Filter->new(
		end=>$etime, start=>($stime-(24*3600*7)), host=>[@hosts]);
	$recordset = $log->fetch($xfilter);
	$ssummary = $recordset->summary($filter);

	mkdir $tmpdir;
	open OUT,">$tmpdir/1.html";
	print OUT hsreport($ssummary);
	close OUT;
	push @files, "$tmpdir/1.html";
	open OUT,">$tmpdir/2.html";
	print OUT ssreport($ssummary);
	close OUT;
	push @files, "$tmpdir/2.html";
	if($q->param('ALT')) {
		open OUT,">$tmpdir/3.html";
		print OUT alertlog($recordset);
		close OUT;
		push @files, "$tmpdir/3.html";
	}

	# Create the PDF, and send it to the client
    $cmd = "$HTMLDOC --webpage --no-links --no-strict --color -t pdf --quiet --path $WEBROOT ".(join " ",@files);

    print $q->header(-target=>'nrbody', -expires=>'now', 
		-type=>'application/pdf');
    select(STDOUT); $|=1;
    system( $cmd );

	# Clean up the mess
	unlink @files;
	rmdir $tmpdir;
}

sub headerform {
	my(@hg);

  $q->param('P','B');

	$cfg = Nagios::Config->new($CFGFILE);
	foreach my $hg ($cfg->list_hostgroups()) {
		push @hg, $hg->name;
	}

	print "<FORM target=nrbody action=".$q->url()." method=post>";
	print "<TABLE width=100% border=0 bgcolor=#cccccc><TR><TD valign=top align=left><H1>Availability Report</H1></TD>"
		."<TD valign=top align=right >"
		.$q->radio_group(-name=>'FMT',-values=>['P','H'],-default=>'H',
			-labels=>{'H'=>'Web page','P'=>'PDF Report'})
		.$q->br.$q->radio_group(-name=>'T',-values=>[0,-1],-default=>0,
			-labels=>{0=>'This Month',-1=>'Last Month'})
		.$q->br.$q->checkbox(-name=>'ALT',-checked=>0,-value=>1,
			-label=>"Include alert log")
		."</TD><TD valign=top align=right ><B>Hostgroup:</B>&nbsp;"
		.$q->popup_menu({-name=>'HG',-values=>[@hg]})
		.$q->hidden({name=>'P', default=>'B'})
#		."</TD><TD valign=top align=right width=1>" 
		.$q->br.$q->submit(-value=>'Generate Report')."</TD></TR>\n";
	print "</TABLE></FORM>\n";
}

sub frameset {
	print "<HTML><HEAD><TITLE>Nagios Hostgroup Report</TITLE></HEAD>\n";
	print "<FRAMESET rows=65,* marginwidth=0>\n";
	print "<FRAME name=nrhead src=".$q->url()."?P=H bgcolor=#cccccc marginwidth=0 topmargin=0 >\n";
	print "<FRAME name=nrbody src=".$q->url()."?P=B&HG=none>\n";
	print "</FRAMESET></HTML>\n";
}

sub blankpage {
	print $q->p("Select a hostgroup and submit.  Note that PDF reports may take several minutes to create if the hostgroup selected has a large number of members.");
}

$hostgroup = $q->param('HG') if($q->param('HG'));

if(!$q->param('P')) {
	print $q->header()."\n";
	frameset();
} elsif( $q->param('P') eq 'H' ) {
	print $q->header()."\n";
	print $q->start_html()."\n";
	headerform();
	print $q->end_html()."\n";
} elsif( $q->param('P') eq 'PDF' or $q->param('FMT') eq 'P' ) {
	makepdf($hostgroup);
} elsif( !$hostgroup ) {
	print $q->header(-target=>'nrbody')."\n";
	print $q->start_html()."\n";
	blankpage;
	print $q->end_html()."\n";
} else {
	print $q->header(-target=>'nrbody')."\n";
	print $q->start_html()."\n";
	hgreport($hostgroup);
	print $q->end_html()."\n";
}


exit(0);
